import {
  Grid, Modal, ModalService, Select, Date as DateInput, AlertService, DialogService,
} from '@zeedhi/common';
import {
  Http, IDictionary, Metadata,
} from '@zeedhi/core';

export class ModalTransfer {
  private PersistentModal!: Modal;

  public async createBasicUsageModal() {
    this.PersistentModal = await ModalService.createFromJson('/modal/modalTransfer', true);
  }

  public destroyBasicUsageModal() {
    this.PersistentModal.destroy();
  }

  public openBasicModal() {
    this.PersistentModal.show();
  }

  public closeBasicModal() {
    this.PersistentModal.hide();
  }

  public setEmployee({ component }:{ component:Select }) {
    component.datasource.data = [];
    Http.get('/Employee').then((response: IDictionary) => {
      component.datasource.data = response.data.dataset.employee;
    });
  }

  public setTask() {
    const funcionarioOrigem = Metadata.getInstance<Select>('SelectFuncionarioOrigem');
    const grid = Metadata.getInstance<Grid>('GridComTarefas');
    grid.datasource.data = [];
    if (funcionarioOrigem.value == null) {
      grid.isVisible = false;
    } else {
      const dataInicio = Metadata.getInstance<DateInput>('InicioPeridoTarefas').value || null;
      const dataFim = Metadata.getInstance<DateInput>('FimPeriodoTarefas').value || null;
      grid.isVisible = true;
      Http.get(`/taskOfTheEmployee?nr=${funcionarioOrigem.value}&inicio=${dataInicio}&fim=${dataFim}`).then((response: IDictionary) => {
        response.data.dataset.task.forEach((element: IDictionary) => {
          grid.datasource.data.push({
            id: element.NRCHECKLISTAGEND,
            name: element.NMCHECKLISTAGEND,
            local: element.NMLOCALCLIENTE,
            date: element.DTHORAAGEND,
          });
        });
      });
    }
  }

  public async transfer() {
    const grid = Metadata.getInstance<Grid>('GridComTarefas');
    const dadosHaAtualizar = grid.selectedRows.map((item: any) => item.id);
    let idVistoria = dadosHaAtualizar.filter((value: any) => value.indexOf('V-') >= 0);
    const idTarefa = dadosHaAtualizar.filter((value: any) => value.indexOf('V-') < 0);
    /* eslint-disable no-console */
    console.log(`Vistoria${idVistoria}\nTarefa: ${idTarefa}`);
    let funcionarioDestino = Metadata.getInstance<Select>('SelectFuncionarioDestino');
    const funcionarioOrigem = Metadata.getInstance<Select>('SelectFuncionarioOrigem').value;
    if (funcionarioDestino.isValid() && (idVistoria.length || idTarefa.length) && (funcionarioDestino.value !== funcionarioOrigem)) {
      funcionarioDestino = funcionarioDestino.value;
      let sucess = true;
      const operadorTransfer = localStorage.getItem('UserLogado');

      if (idVistoria.length) {
        idVistoria = idVistoria.map((value: string) => Number(value.substr(2)));
        await Http.post('/transferVistoria', {
          funcionarioDestino,
          idVistoria,
          operadorTransfer,
          funcionarioOrigem,
        }).then((response: IDictionary) => {
          sucess = response.data.dataset.status[0] === 'OK';
        });
      }

      if (idTarefa.length && sucess) {
        await Http.post('/transferTask', {
          funcionarioDestino,
          idTarefa,
          operadorTransfer,
          funcionarioOrigem,
        }).then((response: IDictionary) => {
          sucess = response.data.dataset.status[0] === 'OK';
        });
      }

      if (sucess) {
        grid.selectedRows = [];
        this.setTask();
        AlertService.show({
          name: 'basic-alert',
          text: 'Tarefas transferida com successo!',
          top: true,
          left: false,
          timeout: 3000,
          color: 'success',
        });
      } else {
        DialogService.show({
          name: 'basic-dialog',
          type: 'warning',
          title: 'Transação bloqueada!',
          text: 'Ocorreu um erro durante a transferencia.\nTente novamente mais tarde.\nSe o erro persistir, entre em contato com o administrador do sistema.',
          buttons: [
            {
              name: 'cancel',
              component: 'ZdButton',
              outline: true,
              label: 'Voltar',
              events: {
                click: () => DialogService.hide(),
              },
            },
          ],
        });
      }
    } else if (!funcionarioDestino.isValid()) {
      AlertService.show({
        name: 'basic-alert',
        text: 'Selecione um funcionário de destino.',
        top: true,
        left: false,
        timeout: 3000,
        color: '#e0ca36',
      });
    } else if (funcionarioDestino.value === funcionarioOrigem) {
      AlertService.show({
        name: 'basic-alert',
        text: 'Selecione um funcionário de destino diferente.',
        top: true,
        left: false,
        timeout: 3000,
        color: '#e0ca36',
      });
    } else {
      AlertService.show({
        name: 'basic-alert',
        text: 'Nenhuma Tarefa selecionada.',
        top: true,
        left: false,
        timeout: 3000,
        color: '#e0ca36',
      });
    }
  }

  public historicoTransfer({ component }:{ component:Grid }) {
    Http.get('/getHistorico').then((response: IDictionary) => {
      component.datasource.data = response.data.dataset.historico;
    });
  }
}
