import {
  Metadata, Loader, Router,
  Http, IDictionary, IEventParam,
} from '@zeedhi/core';
import {
  Text, Container, Form, AlertService,
  TextInput, Select, Grid, MenuGroup,
  Button, Checkbox, DialogService,
} from '@zeedhi/common';

export class PerfilDeAcessoController {
  private ModalPAcessoController = Loader.getInstance('ModalPAcessoController');

  /* -------------------------------- Uso Geral -------------------------------- */

  // Linha selecionada no grid
  public selectedRowsData: IDictionary = [];

  // Fecha a caixa de diálogo
  private cancelDeletesEvents: any = {
    click: () => {
      DialogService.hide();
    },
  };

  // Exclusão Realizada
  public perfAcessDeleted() {
    DialogService.show({
      name: 'positionsDeleted',
      type: 'success',
      title: 'Informações Apagadas',
      text: 'Todas as informações selecionadas foram corretamente apagadas!',
      buttons: [
        {
          name: 'ok',
          component: 'ZdButton',
          label: 'Ok',
          events: this.cancelDeletesEvents,
        },
      ],
    });
  }

  // Verifica qual o acesso e altera a exibição dos elementos
  public isVisibleElement: boolean = !(localStorage.getItem('idpermisssusuario') === 'V');

  public isDisabledElement: boolean = !(this.isVisibleElement);

  // Verifica qual o acesso e altera a exibição do menu
  public menuVisibility() {
    const configuracoesGroup = Metadata.getInstance<MenuGroup>('Configuracoes');
    const configuracoesItems = configuracoesGroup.items;
    const gerencialGroup = Metadata.getInstance<MenuGroup>('gerencial');
    const gerencialItems = gerencialGroup.items;
    const tarefasGroup = Metadata.getInstance<MenuGroup>('tarefas');
    const tarefasItems = tarefasGroup.items;

    // const positionValue = localStorage.getItem('UserLogado');
    // const t1 = Metadata.getInstance<MenuGroup>('tarefas').items;
    // Http.get(`/getInfo/${positionValue}/PositionPermiss`).then((response: IDictionary) => {
    //   const database = response.data.dataset.info[0];

    const idpermissschecklist = localStorage.getItem('idpermissschecklist');
    const idpermisssusuario = localStorage.getItem('idpermisssusuario');
    const idpermisssgraficos = localStorage.getItem('idpermisssgraficos');
    const idpermissslocais = localStorage.getItem('idpermissslocais');
    const idpermisssrelatorios = localStorage.getItem('idpermisssrelatorios');
    const idpermissstarefas = localStorage.getItem('idpermissstarefas');
    const idpermisssvistoria = localStorage.getItem('idpermisssvistoria');
    const idpermissscalend = localStorage.getItem('idpermissscalend');

    if (idpermisssvistoria === 'N' && idpermissscalend === 'N') {
      tarefasGroup.isVisible = false;
    } else {
      if (idpermisssvistoria === 'N') {
        tarefasItems[0].isVisible = false;
      }
      if (idpermissscalend === 'N') {
        tarefasItems[1].isVisible = false;
      }
    }

    if (idpermisssrelatorios === 'N' && idpermisssgraficos === 'N') {
      gerencialGroup.isVisible = false;
    } else {
      if (idpermisssrelatorios === 'N') {
        gerencialItems[1].isVisible = false;
      }
      if (idpermisssgraficos === 'N') {
        gerencialItems[2].isVisible = false;
      }
    }

    if (idpermissslocais === 'N' && idpermissschecklist === 'N' && idpermissstarefas === 'N' && idpermisssusuario === 'N') {
      configuracoesGroup.isVisible = false;
    } else {
      if (idpermissslocais === 'N') {
        configuracoesItems[1].isVisible = false;
      }
      if (idpermissschecklist === 'N') {
        configuracoesItems[2].isVisible = false;
      }
      if (idpermissstarefas === 'N') {
        configuracoesItems[3].isVisible = false;
      }
      if (idpermisssusuario === 'N') {
        configuracoesItems[4].isVisible = false;
      }
    }
    // });
  }

  // Exclusão não realizada
  public perfAcessNotDeleted() {
    DialogService.show({
      name: 'positionsDeleted',
      type: 'error',
      title: 'Informações não apagadas',
      text: 'Algumas informações não podem ser apagadas, pois possuem dependências!',
      buttons: [
        {
          name: 'ok',
          component: 'ZdButton',
          label: 'Ok',
          events: this.cancelDeletesEvents,
        },
      ],
    });
  }

  // Busca pelas permissões do usuário
  public getPermissions() {
    // return new Promise((resolve, reject) => {
    //   try {
    const userId = localStorage.getItem('UserLogado');

    Http.get(`/getInfo/${userId}/PositionPermiss`).then((response: IDictionary) => {
      const database = response.data.dataset.info[0];
      if (localStorage.getItem('Token') === database.token) {
        window.localStorage.setItem('idpermisssusuario', database.IDPERMISSSUSUARIO);
        window.localStorage.setItem('idpermissstarefas', database.IDPERMISSSTAREFAS);
        window.localStorage.setItem('idpermissslocais', database.IDPERMISSSLOCAIS);
        window.localStorage.setItem('idpermissschecklist', database.IDPERMISSSCHECKLIST);
        window.localStorage.setItem('idpermisssrelatorios', database.IDPERMISSSRELATORIOS);
        window.localStorage.setItem('idpermisssgraficos', database.IDPERMISSSGRAFICOS);
        window.localStorage.setItem('idpermisssvistoria', database.IDPERMISSSVISTORIA);
        window.localStorage.setItem('idpermissscalend', database.IDPERMISSSCALEND);
      } else {
        window.localStorage.clear();
        Router.replace('/Login');
      }
    });
    //     resolve();
    //   } catch (error) {
    //     reject(error);
    //   }
    // });
  }

  /* -------------------------------- Setores -------------------------------- */

  // data do grid Setores
  public dataSector(nrSector: number, nmSector: string, nrFuncresponsavel: number, nmResponsavel: string) {
    return {
      sector: nmSector,
      responsible: nmResponsavel,
      idReponsible: nrFuncresponsavel,
      value: nrSector,
    };
  }

  // data do Grid de locais vinculados
  public dataLinkedLocals(nrLocal: number, nmLocal: string) {
    return {
      value: nrLocal,
      local: nmLocal,
    };
  }

  // Direciona para o evento de delete
  private deleteSectorsEvents: any = {
    click: () => {
      this.deleteSectors();
      DialogService.hide();
    },
  };

  // Abrir modal Novo Setor
  public openModalSetor() {
    this.ModalPAcessoController.openModalSetor();
  }

  // Abrir modal Editar usuário
  public openToUpdateModalSetor() {
    Promise.all([this.openModalSetor()]).then(() => {
      const textObject = Metadata.getInstance<Text>('newSectorTitle');
      const nameInputObject = Metadata.getInstance<TextInput>('sectorName');
      const responsibleInputObject = Metadata.getInstance<Select>('userSelect');
      const saveButtonObject = Metadata.getInstance<Button>('sectorSaveButton');
      const cancelButtonObject = Metadata.getInstance<Button>('sectorCancelButton');

      nameInputObject.disabled = this.isDisabledElement;
      responsibleInputObject.disabled = this.isDisabledElement;
      saveButtonObject.disabled = this.isDisabledElement;
      cancelButtonObject.disabled = this.isDisabledElement;

      textObject.text = 'Dados do Setor';

      const containerObject = Metadata.getInstance<Container>('updateSectorContainer');
      containerObject.isVisible = true;

      const gridObject = Metadata.getInstance<Grid>('sectorsGrid');
      const sectorInfos = gridObject.datasource.currentRow;
      this.setSectorToUpdate(sectorInfos);
      this.getLinkedLocals(sectorInfos);
    });
  }

  // Exibe setores
  public getSectors() {
    const gridObject = Metadata.getInstance<Grid>('sectorsGrid');
    gridObject.datasource.data = [];

    Http.post('/getSectors', this.dataFilter).then((response: IDictionary) => {
      response.data.dataset.info.forEach((database: IDictionary) => {
        gridObject.datasource.data.push(this.dataSector(database.NRSETOR, database.NMSETOR, database.NRFUNCRESPONSAVEL, database.NMFUNCIONARIO));
      });
    });
  }

  // Valida sectorForm
  public sectorFormValidation() {
    const formObject = Metadata.getInstance<Form>('newSectorForm');
    const sectorTitle = Metadata.getInstance<Text>('newSectorTitle');

    if (formObject.validate()) {
      if (sectorTitle.text === 'Novo Setor') {
        this.createSector();
      } else {
        this.updateSector();
      }
    } else {
      AlertService.show({
        name: 'buttons-alert',
        text: 'Preencha todos os campos',
        color: 'error',
      });
    }
  }

  // Cria um novo setor
  public createSector() {
    const sectorInfos = {
      NMSETOR: Metadata.getInstance<TextInput>('sectorName').value,
      NRFUNCRESPONSAVEL: Metadata.getInstance<Select>('userSelect').value,
    };
    Http.post('/createSector', { sectorInfos });
    this.ModalPAcessoController.closeModalSetor();
  }

  // Habilita botões criar/editar/deletar
  public sectorRowEvent({ component }: IEventParam<Grid>) {
    const editButtonObject = Metadata.getInstance<Button>('sectorEdit');
    const deleteButtonObject = Metadata.getInstance<Button>('sectorDelete');
    const createButtonObject = Metadata.getInstance<Button>('sectorAdd');
    const rowsLength = component.selectedRows.length;
    const rowsObject = component.selectedRows.map((item: any) => item.value);
    this.selectedRowsData = rowsObject;

    if (rowsLength === 1) {
      editButtonObject.disabled = false;
    } else editButtonObject.disabled = true;

    if (rowsLength > 0) {
      deleteButtonObject.disabled = false;
      createButtonObject.disabled = true;
    } else {
      deleteButtonObject.disabled = true;
      createButtonObject.disabled = false;
    }
  }

  // Busca as info do setor para editar
  public setSectorToUpdate(infos: IDictionary) {
    const params = {
      NRSETOR: infos.value,
    };

    Http.post('/getSectorInfos', { params }).then((response: IDictionary) => {
      response.data.dataset.info.forEach((database: IDictionary) => {
        Metadata.getInstance<Text>('nrsetorST').text = database.NRSETOR;
        Metadata.getInstance<TextInput>('sectorName').value = database.NMSETOR;
        Metadata.getInstance<Select>('userSelect').setValue(database.NRFUNCRESPONSAVEL);
      });
    });
  }

  // Edita setor
  public updateSector() {
    const params = {
      NRSETOR: Metadata.getInstance<Text>('nrsetorST').text,
      NMSETOR: Metadata.getInstance<TextInput>('sectorName').value,
      NRFUNCRESPONSAVEL: Metadata.getInstance<Select>('userSelect').value,
    };
    Http.post('/updateSector', { params });
    this.getSectors();
    this.ModalPAcessoController.closeModalSetor();
  }

  // Deleta setores
  public deleteSectors() {
    let validResponse: number;
    const params = this.selectedRowsData;

    Http.post('/deleteSectors', { params }).then((response: IDictionary) => {
      validResponse = response.data.dataset.info.resultado;
    }).then(() => {
      if (validResponse === 0) this.perfAcessDeleted();
      else this.perfAcessNotDeleted();

      this.getSectors();
    });
  }

  // Caixa de confirmação de delete
  public confirmSectorsDelete() {
    DialogService.show({
      name: 'button-delete',
      type: 'warning',
      title: 'Excluir Setor',
      text: 'Todas as informações deste(s) Setor(es) serão perdidas. Deseja continuar?',
      buttons: [
        {
          name: 'yes',
          component: 'ZdButton',
          label: 'Sim',
          events: this.deleteSectorsEvents,
          outline: true,
        },
        {
          name: 'no',
          component: 'ZdButton',
          label: 'Não',
          events: this.cancelDeletesEvents,
        },
      ],
    });
  }

  // Busca por setores
  public fillSectorFilter({ component }: IEventParam<TextInput>) {
    this.dataFilter[component.name] = component.value;
    this.getSectors();
  }

  // Busca Por locais Vinculados ao setor
  public getLinkedLocals(info: IDictionary) {
    const gridObject = Metadata.getInstance<Grid>('sectorLocalGrid');
    const params = info.value;

    Http.get(`/getLinkedLocals/${params}/locals`).then((response: IDictionary) => {
      response.data.dataset.info.forEach((database: IDictionary) => {
        gridObject.datasource.data.push(this.dataLinkedLocals(database.NRLOCALCLIENTE, database.NMLOCALCLIENTE));
      });
    });
  }

  /* -------------------------------- Funções -------------------------------- */

  // Data Lista  de Funções
  public dataPosition(infoValue: number, infoPosition: string) {
    return {
      position: infoPosition,
      permissions: 'mdi-plus',
      value: infoValue,
    };
  }

  // Direciona para o evento de delete
  private deletePositionsEvents: any = {
    click: () => {
      this.deletePositions();
      DialogService.hide();
    },
  };

  // input data pesquisar
  public dataFilter: IDictionary = {
    positionSearch: '',
    sectorSearch: '',
    userSearch: '',
  };

  // Condições das checkbox para o BD
  public readOrWriteData: IDictionary = {
    users: '',
    inspections: '',
    tasks: '',
    locals: '',
    checklists: '',
    reports: '',
    graphs: '',
    calendar: '',
  };

  // Abrir modal Nova Função
  public openModalFuncao() {
    this.ModalPAcessoController.openModalFuncao();
  }

  // Abrir modal Editar Função (Não está sendo mais usado)
  public openToUpdateModalFuncao() {
    Promise.all([this.openModalFuncao()]).then(() => {
      const gridObject = Metadata.getInstance<Grid>('positionsGrid');
      const textObject = Metadata.getInstance<Text>('newPositionTitle');
      const saveButtonObject = Metadata.getInstance<Button>('positionSaveButton');
      const cancelButtonObject = Metadata.getInstance<Button>('positionCancelButton');

      Metadata.getInstance<TextInput>('positionName').disabled = this.isDisabledElement;
      Metadata.getInstance<Select>('positionDashboard').disabled = this.isDisabledElement;
      Metadata.getInstance<Select>('positionQRCode').disabled = this.isDisabledElement;

      Metadata.getInstance<Checkbox>('usersVCheck').disabled = this.isDisabledElement;
      Metadata.getInstance<Checkbox>('usersECheck').disabled = this.isDisabledElement;

      Metadata.getInstance<Checkbox>('inspectionsVCheck').disabled = this.isDisabledElement;
      Metadata.getInstance<Checkbox>('inspectionsECheck').disabled = this.isDisabledElement;

      Metadata.getInstance<Checkbox>('tasksVCheck').disabled = this.isDisabledElement;
      Metadata.getInstance<Checkbox>('tasksECheck').disabled = this.isDisabledElement;

      Metadata.getInstance<Checkbox>('localsVCheck').disabled = this.isDisabledElement;
      Metadata.getInstance<Checkbox>('localsECheck').disabled = this.isDisabledElement;

      Metadata.getInstance<Checkbox>('checklistsVCheck').disabled = this.isDisabledElement;
      Metadata.getInstance<Checkbox>('checklistsECheck').disabled = this.isDisabledElement;

      Metadata.getInstance<Checkbox>('reportsVCheck').disabled = this.isDisabledElement;
      Metadata.getInstance<Checkbox>('reportsECheck').disabled = this.isDisabledElement;

      Metadata.getInstance<Checkbox>('graphsVCheck').disabled = this.isDisabledElement;
      Metadata.getInstance<Checkbox>('graphsECheck').disabled = this.isDisabledElement;

      Metadata.getInstance<Checkbox>('calendarVCheck').disabled = this.isDisabledElement;
      Metadata.getInstance<Checkbox>('calendarECheck').disabled = this.isDisabledElement;

      saveButtonObject.disabled = this.isDisabledElement;
      cancelButtonObject.disabled = this.isDisabledElement;

      textObject.text = 'Dados da Função';

      const positionInfos = gridObject.datasource.currentRow;
      this.setPositionToUpdate(positionInfos);
    });
  }

  // Não estou usando
  // public visualizePosition() {
  //   Promise.all([this.openModalFuncao()]).then(() => {
  //     const gridObject = Metadata.getInstance<Grid>('positionsGrid');
  //     const textObject = Metadata.getInstance<Text>('newPositionTitle');
  //     const saveButtonObject = Metadata.getInstance<Button>('positionSaveButton');
  //     const cancelButtonObject = Metadata.getInstance<Button>('positionCancelButton');

  //     Metadata.getInstance<TextInput>('positionName').disabled = this.isDisabledElement;
  //     Metadata.getInstance<Select>('positionDashboard').disabled = this.isDisabledElement;
  //     Metadata.getInstance<Select>('positionQRCode').disabled = this.isDisabledElement;

  //     Metadata.getInstance<Checkbox>('usersVCheck').disabled = this.isDisabledElement;
  //     Metadata.getInstance<Checkbox>('usersECheck').disabled = this.isDisabledElement;

  //     Metadata.getInstance<Checkbox>('inspectionsVCheck').disabled = this.isDisabledElement;
  //     Metadata.getInstance<Checkbox>('inspectionsECheck').disabled = this.isDisabledElement;

  //     Metadata.getInstance<Checkbox>('tasksVCheck').disabled = this.isDisabledElement;
  //     Metadata.getInstance<Checkbox>('tasksECheck').disabled = this.isDisabledElement;

  //     Metadata.getInstance<Checkbox>('localsVCheck').disabled = this.isDisabledElement;
  //     Metadata.getInstance<Checkbox>('localsECheck').disabled = this.isDisabledElement;

  //     Metadata.getInstance<Checkbox>('checklistsVCheck').disabled = this.isDisabledElement;
  //     Metadata.getInstance<Checkbox>('checklistsECheck').disabled = this.isDisabledElement;

  //     Metadata.getInstance<Checkbox>('reportsVCheck').disabled = this.isDisabledElement;
  //     Metadata.getInstance<Checkbox>('reportsECheck').disabled = this.isDisabledElement;

  //     Metadata.getInstance<Checkbox>('graphsVCheck').disabled = this.isDisabledElement;
  //     Metadata.getInstance<Checkbox>('graphsECheck').disabled = this.isDisabledElement;

  //     Metadata.getInstance<Checkbox>('calendarVCheck').disabled = this.isDisabledElement;
  //     Metadata.getInstance<Checkbox>('calendarECheck').disabled = this.isDisabledElement;

  //     textObject.text = 'Dados da Função';
  //     saveButtonObject.isVisible = false;
  //     cancelButtonObject.isVisible = false;

  //     const positionInfos = gridObject.datasource.currentRow;
  //     this.setPositionToUpdate(positionInfos);
  //   });
  // }

  // Exibir Funções
  public getPositions() {
    const gridObject = Metadata.getInstance<Grid>('positionsGrid');
    gridObject.datasource.data = [];

    Http.post('/getPositions', this.dataFilter).then((response: IDictionary) => {
      response.data.dataset.info.forEach((database: IDictionary) => {
        gridObject.datasource.data.push(this.dataPosition(database.NRPERFILACESSO, database.NMPERFILACESSO));
      });
    });
  }

  // Preenche array de acordo com o valor do array acima
  public databasePermissions() {
    const usersVObject = Metadata.getInstance<Checkbox>('usersVCheck').value;
    const usersEObject = Metadata.getInstance<Checkbox>('usersECheck').value;

    const inspectionsVObject = Metadata.getInstance<Checkbox>('inspectionsVCheck').value;
    const inspectionsEObject = Metadata.getInstance<Checkbox>('inspectionsECheck').value;

    const tasksVObject = Metadata.getInstance<Checkbox>('tasksVCheck').value;
    const tasksEObject = Metadata.getInstance<Checkbox>('tasksECheck').value;

    const localsVObject = Metadata.getInstance<Checkbox>('localsVCheck').value;
    const localsEObject = Metadata.getInstance<Checkbox>('localsECheck').value;

    const checklistsVObject = Metadata.getInstance<Checkbox>('checklistsVCheck').value;
    const checklistsEObject = Metadata.getInstance<Checkbox>('checklistsECheck').value;

    const reportsVObject = Metadata.getInstance<Checkbox>('reportsVCheck').value;
    const reportsEObject = Metadata.getInstance<Checkbox>('reportsECheck').value;

    const graphsVObject = Metadata.getInstance<Checkbox>('graphsVCheck').value;
    const graphsEObject = Metadata.getInstance<Checkbox>('graphsECheck').value;

    const calendarVObject = Metadata.getInstance<Checkbox>('calendarVCheck').value;
    const calendarEObject = Metadata.getInstance<Checkbox>('calendarECheck').value;

    if ((usersVObject && usersEObject) || usersEObject) {
      this.readOrWriteData.users = 'E';
    } else if (usersVObject) {
      this.readOrWriteData.users = 'V';
    } else this.readOrWriteData.users = 'N';

    if ((inspectionsVObject && inspectionsEObject) || inspectionsEObject) {
      this.readOrWriteData.inspections = 'E';
    } else if (inspectionsVObject) {
      this.readOrWriteData.inspections = 'V';
    } else this.readOrWriteData.inspections = 'N';

    if ((tasksVObject && tasksEObject) || tasksEObject) {
      this.readOrWriteData.tasks = 'E';
    } else if (tasksVObject) {
      this.readOrWriteData.tasks = 'V';
    } else this.readOrWriteData.tasks = 'N';

    if ((localsVObject && localsEObject) || localsEObject) {
      this.readOrWriteData.locals = 'E';
    } else if (localsVObject) {
      this.readOrWriteData.locals = 'V';
    } else this.readOrWriteData.locals = 'N';

    if ((checklistsVObject && checklistsEObject) || checklistsEObject) {
      this.readOrWriteData.checklists = 'E';
    } else if (checklistsVObject) {
      this.readOrWriteData.checklists = 'V';
    } else this.readOrWriteData.checklists = 'N';

    if ((reportsVObject && reportsEObject) || reportsEObject) {
      this.readOrWriteData.reports = 'E';
    } else if (reportsVObject) {
      this.readOrWriteData.reports = 'V';
    } else this.readOrWriteData.reports = 'N';

    if ((graphsVObject && graphsEObject) || graphsEObject) {
      this.readOrWriteData.graphs = 'E';
    } else if (graphsVObject) {
      this.readOrWriteData.graphs = 'V';
    } else this.readOrWriteData.graphs = 'N';

    if ((calendarVObject && calendarEObject) || calendarEObject) {
      this.readOrWriteData.calendar = 'E';
    } else if (calendarVObject) {
      this.readOrWriteData.calendar = 'V';
    } else this.readOrWriteData.calendar = 'N';
  }

  // Prenche o array para edição
  public setPermissions(idpermisssusuario: string, idpermisssvistoria: string, idpermissstarefas: string, idpermissslocais: string, idpermissschecklist: string, idpermisssrelatorios: string, idpermisssgraficos: string, idpermissscalend: string) {
    const usersVObject = Metadata.getInstance<Checkbox>('usersVCheck');
    const usersEObject = Metadata.getInstance<Checkbox>('usersECheck');

    const inspectionsVObject = Metadata.getInstance<Checkbox>('inspectionsVCheck');
    const inspectionsEObject = Metadata.getInstance<Checkbox>('inspectionsECheck');

    const tasksVObject = Metadata.getInstance<Checkbox>('tasksVCheck');
    const tasksEObject = Metadata.getInstance<Checkbox>('tasksECheck');

    const localsVObject = Metadata.getInstance<Checkbox>('localsVCheck');
    const localsEObject = Metadata.getInstance<Checkbox>('localsECheck');

    const checklistsVObject = Metadata.getInstance<Checkbox>('checklistsVCheck');
    const checklistsEObject = Metadata.getInstance<Checkbox>('checklistsECheck');

    const reportsVObject = Metadata.getInstance<Checkbox>('reportsVCheck');
    const reportsEObject = Metadata.getInstance<Checkbox>('reportsECheck');

    const graphsVObject = Metadata.getInstance<Checkbox>('graphsVCheck');
    const graphsEObject = Metadata.getInstance<Checkbox>('graphsECheck');

    const calendarVObject = Metadata.getInstance<Checkbox>('calendarVCheck');
    const calendarEObject = Metadata.getInstance<Checkbox>('calendarECheck');

    if (idpermisssusuario === 'A' || idpermisssusuario === 'E') {
      usersVObject.value = true;
      usersEObject.value = true;
    } else if (idpermisssusuario === 'V') {
      usersVObject.value = true;
    } else {
      usersVObject.value = false;
      usersEObject.value = false;
    }

    if (idpermisssvistoria === 'A' || idpermisssvistoria === 'E') {
      inspectionsVObject.value = true;
      inspectionsEObject.value = true;
    } else if (idpermisssvistoria === 'V') {
      inspectionsVObject.value = true;
    } else {
      inspectionsVObject.value = false;
      inspectionsEObject.value = false;
    }

    if (idpermissstarefas === 'A' || idpermissstarefas === 'E') {
      tasksVObject.value = true;
      tasksEObject.value = true;
    } else if (idpermissstarefas === 'V') {
      tasksVObject.value = true;
    } else {
      tasksVObject.value = false;
      tasksEObject.value = false;
    }

    if (idpermissslocais === 'A' || idpermissslocais === 'E') {
      localsVObject.value = true;
      localsEObject.value = true;
    } else if (idpermissslocais === 'V') {
      localsVObject.value = true;
    } else {
      localsVObject.value = false;
      localsEObject.value = false;
    }

    if (idpermissschecklist === 'A' || idpermissschecklist === 'E') {
      checklistsVObject.value = true;
      checklistsEObject.value = true;
    } else if (idpermissschecklist === 'V') {
      checklistsVObject.value = true;
    } else {
      checklistsVObject.value = false;
      checklistsEObject.value = false;
    }

    if (idpermisssrelatorios === 'A' || idpermisssrelatorios === 'E') {
      reportsVObject.value = true;
      reportsEObject.value = true;
    } else if (idpermisssrelatorios === 'V') {
      reportsVObject.value = true;
    } else {
      reportsVObject.value = false;
      reportsEObject.value = false;
    }

    if (idpermisssgraficos === 'A' || idpermisssgraficos === 'E') {
      graphsVObject.value = true;
      graphsEObject.value = true;
    } else if (idpermisssgraficos === 'V') {
      graphsVObject.value = true;
    } else {
      graphsVObject.value = false;
      graphsEObject.value = false;
    }

    if (idpermissscalend === 'A' || idpermissscalend === 'E') {
      calendarVObject.value = true;
      calendarEObject.value = true;
    } else if (idpermissscalend === 'V') {
      calendarVObject.value = true;
    } else {
      calendarVObject.value = false;
      calendarEObject.value = false;
    }
  }

  // Cria uma nova Função
  public createPosition() {
    this.databasePermissions();
    const params = {
      NMPERFILACESSO: Metadata.getInstance<TextInput>('positionName').value,
      IDPERMISSSUSUARIO: this.readOrWriteData.users,
      IDPERMISSSFUNCOES: this.readOrWriteData.positions,
      IDPERMISSSTAREFAS: this.readOrWriteData.tasks,
      IDPERMISSSLOCAIS: this.readOrWriteData.locals,
      IDPERMISSSCHECKLIST: this.readOrWriteData.checklists,
      IDPERMISSSRELATORIOS: this.readOrWriteData.reports,
      IDPERMISSSGRAFICOS: this.readOrWriteData.graphs,
      IDPERMISSSVISTORIA: this.readOrWriteData.inspections,
      IDPERMISSSCALEND: this.readOrWriteData.calendar,
      IDDIRECIONAQRCODE: Metadata.getInstance<Select>('positionQRCode').value,
      IDTIPODASHBOARD: Metadata.getInstance<Select>('positionDashboard').value,
    };
    Http.post('/createPosition', { params });
    this.getPositions();
    this.ModalPAcessoController.closeModalFuncao();
  }

  // Seta os dados no modal para editar
  public setPositionToUpdate(infos: IDictionary) {
    const params = infos.value;

    Http.get(`/position/${params}/info`).then((response: IDictionary) => {
      response.data.dataset.info.forEach((database: IDictionary) => {
        Metadata.getInstance<TextInput>('positionName').value = database.NMPERFILACESSO;
        Metadata.getInstance<Text>('nrperfilacesso').text = database.NRPERFILACESSO;
        Metadata.getInstance<Select>('positionQRCode').setValue(database.IDDIRECIONAQRCODE);
        Metadata.getInstance<Select>('positionDashboard').setValue(database.IDTIPODASHBOARD);
        this.setPermissions(database.IDPERMISSSUSUARIO, database.IDPERMISSSVISTORIA, database.IDPERMISSSTAREFAS, database.IDPERMISSSLOCAIS, database.IDPERMISSSCHECKLIST, database.IDPERMISSSRELATORIOS, database.IDPERMISSSGRAFICOS, database.IDPERMISSSCALEND);
      });
    });
  }

  // Edita a função
  public updatePosition() {
    this.databasePermissions();
    const params = {
      NRPERFILACESSO: Metadata.getInstance<Text>('nrperfilacesso').text,
      NMPERFILACESSO: Metadata.getInstance<TextInput>('positionName').value,
      IDPERMISSSUSUARIO: this.readOrWriteData.users,
      IDPERMISSSFUNCOES: this.readOrWriteData.positions,
      IDPERMISSSTAREFAS: this.readOrWriteData.tasks,
      IDPERMISSSLOCAIS: this.readOrWriteData.locals,
      IDPERMISSSCHECKLIST: this.readOrWriteData.checklists,
      IDPERMISSSRELATORIOS: this.readOrWriteData.reports,
      IDPERMISSSGRAFICOS: this.readOrWriteData.graphs,
      IDPERMISSSVISTORIA: this.readOrWriteData.inspections,
      IDPERMISSSCALEND: this.readOrWriteData.calendar,
      IDDIRECIONAQRCODE: Metadata.getInstance<Select>('positionQRCode').value,
      IDTIPODASHBOARD: Metadata.getInstance<Select>('positionDashboard').value,
    };
    Http.post('/updatePosition', { params });
    this.getPositions();
    this.ModalPAcessoController.closeModalFuncao();
  }

  // Deleta a função
  public deletePositions() {
    let validResponse: number;
    const params = this.selectedRowsData;

    Http.post('/deletePositions', { params }).then((response: IDictionary) => {
      validResponse = response.data.dataset.info.resultado;
    }).then(() => {
      this.getPositions();

      if (validResponse === 0) this.perfAcessDeleted();
      else this.perfAcessNotDeleted();
    });
  }

  // Habilita os botôes de editar e deletar
  public positionRowEvent({ component }: IEventParam<Grid>) {
    const deleteButtonObject = Metadata.getInstance<Button>('positionDelete');
    const createButtonObject = Metadata.getInstance<Button>('positionAdd');

    const rowsLength = component.selectedRows.length;
    const rowsObject = component.selectedRows.map((item: any) => item.value);
    this.selectedRowsData = rowsObject;

    if (rowsLength > 0) {
      deleteButtonObject.disabled = false;
      createButtonObject.disabled = true;
    } else {
      deleteButtonObject.disabled = true;
      createButtonObject.disabled = false;
    }
  }

  // Confirmar se realiza a exclusão
  public confirmPositionsDelete() {
    DialogService.show({
      name: 'button-delete',
      type: 'warning',
      title: 'Excluir Função',
      text: 'Todas as informações desta(s) Função(ões) serão perdidas. Deseja continuar?',
      buttons: [
        {
          name: 'yes',
          component: 'ZdButton',
          label: 'Sim',
          events: this.deletePositionsEvents,
          outline: true,
        },
        {
          name: 'no',
          component: 'ZdButton',
          label: 'Não',
          events: this.cancelDeletesEvents,
        },
      ],
    });
  }

  // Pesquisa a Função
  public fillPositionFilter({ component }: IEventParam<TextInput>) {
    this.dataFilter[component.name] = component.value;
    this.getPositions();
  }

  // Valida formulário Nova função
  public validatePositionForm() {
    const formObject = Metadata.getInstance<Form>('newPositionForm');
    const titleText = Metadata.getInstance<Text>('newPositionTitle');

    if (formObject.validate()) {
      if (titleText.text === 'Nova Função') {
        this.createPosition();
      } else {
        this.updatePosition();
      }
    } else {
      AlertService.show({
        name: 'buttons-alert',
        text: 'Preencha todos os campos.',
        color: 'error',
      });
    }
  }

  // Função de teste de variáveis
  public consoleLog(variavel: any) {
    const titulo = Metadata.getInstance<Text>('titulo');
    titulo.text = variavel;
  }

  /* -------------------------------- Usuários -------------------------------- */

  // Infos data Usuários
  public dataUser(infoValue: number, infoUser: string, infoEmail: string, infoTel: string, infoPassword: string, nmPerfilacesso: string, nmSetor: String) {
    return {
      user: infoUser,
      position: nmPerfilacesso,
      email: infoEmail,
      tel: infoTel,
      password: infoPassword,
      sector: nmSetor,
      value: infoValue,
    };
  }

  // Direciona para a função de delete
  private deleteUsersEvents: any = {
    click: () => {
      this.deleteUsers();
      DialogService.hide();
    },
  };

  // Abrir modal Novo Usuário
  public openModalUsuario() {
    this.ModalPAcessoController.openModalUsuario();
  }

  // Habilita os botôes de editar e excluir
  public userRowEvent({ component }: IEventParam<Grid>) {
    const editButtonObject = Metadata.getInstance<Button>('userEdit');
    const deleteButtonObject = Metadata.getInstance<Button>('userDelete');
    const createButtonObject = Metadata.getInstance<Button>('userAdd');
    const rowsLength = component.selectedRows.length;
    const rowsObject = component.selectedRows.map((item: any) => item.value);
    this.selectedRowsData = rowsObject;

    if (rowsLength === 1) {
      editButtonObject.disabled = false;
    } else editButtonObject.disabled = true;

    if (rowsLength > 0) {
      deleteButtonObject.disabled = false;
      createButtonObject.disabled = true;
    } else {
      deleteButtonObject.disabled = true;
      createButtonObject.disabled = false;
    }
  }

  // Abrir modal Editar Usuário
  public openToUpdateModalUsuario() {
    Promise.all([this.openModalUsuario()]).then(() => {
      const textObject = Metadata.getInstance<Text>('titleTextNewEmployee');
      const gridObject = Metadata.getInstance<Grid>('usersGrid');
      textObject.text = 'Dados do Usuário';

      Metadata.getInstance<TextInput>('userName').disabled = this.isDisabledElement;
      Metadata.getInstance<Select>('positionSelect').disabled = this.isDisabledElement;
      Metadata.getInstance<TextInput>('userCPF').disabled = this.isDisabledElement;
      Metadata.getInstance<TextInput>('userRG').disabled = this.isDisabledElement;
      Metadata.getInstance<TextInput>('userEmail').disabled = this.isDisabledElement;
      Metadata.getInstance<TextInput>('userTel').disabled = this.isDisabledElement;
      Metadata.getInstance<TextInput>('userPassword').disabled = this.isDisabledElement;
      Metadata.getInstance<Select>('sectorSelect').disabled = this.isDisabledElement;
      Metadata.getInstance<Button>('userCancelButton').disabled = this.isDisabledElement;
      Metadata.getInstance<Button>('userSaveButton').disabled = this.isDisabledElement;

      const userInfos = gridObject.datasource.currentRow;
      this.setUserToUpdate(userInfos);
    });
  }

  // Exibir usuários
  public getUsers() {
    const gridObject = Metadata.getInstance<Grid>('usersGrid');
    gridObject.datasource.data = [];

    Http.post('/getUsers', this.dataFilter).then((response: IDictionary) => {
      response.data.dataset.info.forEach((database: IDictionary) => {
        gridObject.datasource.data.push(this.dataUser(database.NRFUNCIONARIO, database.NMFUNCIONARIO, database.DSEMAILFUNCIONARIO, database.NRTELFUNCIONARIO, database.DSSENHAFUNC, database.NMPERFILACESSO, database.NMSETOR));
      });
    });
  }

  // Valida userForm
  public userFormValidation() {
    const formObject = Metadata.getInstance<Form>('newUserForm');
    const textObject = Metadata.getInstance<Text>('titleTextNewEmployee');

    if (formObject.validate()) {
      if (textObject.text === 'Novo Usuário') {
        this.createUser();
      } else {
        this.updateUser();
      }
    } else {
      AlertService.show({
        name: 'buttons-alert',
        text: 'Preencha todos os campos',
        color: 'error',
      });
    }
  }

  // Criar novo Usuário
  public createUser() {
    const userInfos = {
      NMFUNCIONARIO: Metadata.getInstance<TextInput>('userName').value,
      NRPERFILACESSO: Metadata.getInstance<Select>('positionSelect').value,
      NRCPFFUNCIONARIO: Metadata.getInstance<TextInput>('userCPF').value,
      NRRGFUNCIONARIO: Metadata.getInstance<TextInput>('userRG').value,
      DSEMAILFUNCIONARIO: Metadata.getInstance<TextInput>('userEmail').value,
      NRTELFUNCIONARIO: Metadata.getInstance<TextInput>('userTel').value,
      DSSENHAFUNC: Metadata.getInstance<TextInput>('userPassword').value,
      NRSETOR: Metadata.getInstance<Select>('sectorSelect').value,
    };
    Http.post('/createUser', { userInfos });
    this.ModalPAcessoController.closeModalUsuario();
  }

  // Seta nos campos do modal usuário e altera evento de click para update
  public setUserToUpdate(infos: IDictionary) {
    const params = {
      NRFUNCIONARIO: infos.value,
    };

    Http.post('/getUserInfos', { params }).then((response: IDictionary) => {
      response.data.dataset.info.forEach((database: IDictionary) => {
        Metadata.getInstance<Text>('nrfuncionario').text = database.NRFUNCIONARIO;
        Metadata.getInstance<TextInput>('userName').value = database.NMFUNCIONARIO;
        Metadata.getInstance<Select>('positionSelect').setValue(database.NRPERFILACESSO);
        Metadata.getInstance<Text>('relacfuncperfacess').text = database.RELACFUNCPERFACESS;
        Metadata.getInstance<TextInput>('userCPF').value = database.NRCPFFUNCIONARIO;
        Metadata.getInstance<TextInput>('userRG').value = database.NRRGFUNCIONARIO;
        Metadata.getInstance<TextInput>('userEmail').value = database.DSEMAILFUNCIONARIO;
        Metadata.getInstance<TextInput>('userTel').value = (database.NRTELFUNCIONARIO).toString();
        Metadata.getInstance<TextInput>('userPassword').value = database.DSSENHAFUNC;
        Metadata.getInstance<Select>('sectorSelect').setValue(database.NRSETOR);
        Metadata.getInstance<Text>('nrrelacfuncsetor').text = database.NRRELACFUNCSETOR;
      });
    });
  }

  // Editar usuário
  public updateUser() {
    const params = {
      NRFUNCIONARIO: Metadata.getInstance<Text>('nrfuncionario').text,
      NMFUNCIONARIO: Metadata.getInstance<TextInput>('userName').value,
      NRPERFILACESSO: Metadata.getInstance<Select>('positionSelect').value,
      NRCPFFUNCIONARIO: Metadata.getInstance<TextInput>('userCPF').value,
      NRRGFUNCIONARIO: Metadata.getInstance<TextInput>('userRG').value,
      DSEMAILFUNCIONARIO: Metadata.getInstance<TextInput>('userEmail').value,
      NRTELFUNCIONARIO: Metadata.getInstance<TextInput>('userTel').value,
      DSSENHAFUNC: Metadata.getInstance<TextInput>('userPassword').value,
      NRSETOR: Metadata.getInstance<Select>('sectorSelect').value,
      RELACFUNCPERFACESS: Metadata.getInstance<Text>('relacfuncperfacess').text,
      NRRELACFUNCSETOR: Metadata.getInstance<Text>('nrrelacfuncsetor').text,
    };
    Http.put('/updateUser', { params });
    this.getUsers();
    this.ModalPAcessoController.closeModalUsuario();
  }

  // Deletar Usuário
  public deleteUsers() {
    let validResponse: number;
    const params = this.selectedRowsData;

    Http.post('/deleteUsers', { params }).then((response: IDictionary) => {
      validResponse = response.data.dataset.info.resultado;
    }).then(() => {
      if (validResponse === 0) this.perfAcessDeleted();
      else this.perfAcessNotDeleted();

      this.getUsers();
    });
  }

  // Confirmar se será feita a exclusão
  public confirmUserDelete() {
    DialogService.show({
      name: 'button-delete',
      type: 'warning',
      title: 'Excluir Usuário',
      text: 'Todas as informações deste(s) Usuário(s) serão perdidas. Deseja continuar?',
      buttons: [
        {
          name: 'yes',
          component: 'ZdButton',
          label: 'Sim',
          events: this.deleteUsersEvents,
          outline: true,
        },
        {
          name: 'no',
          component: 'ZdButton',
          label: 'Não',
          events: this.cancelDeletesEvents,
        },
      ],
    });
  }

  // Busca por usuários
  public fillUserFilter({ component }: IEventParam<TextInput>) {
    this.dataFilter[component.name] = component.value;
    this.getUsers();
  }
}
